@@ -379,11 +379,11 @@ void GjsCallbackTrampoline::callback_closure(GIArgument** args, void* result) {
             // main loop, or maybe not, but there's no way to tell, so we have
             // to exit here instead of propagating the exception back to the
             // original calling JS code.
-            gjs->warn_about_unhandled_promise_rejections();
-
             uint8_t code;
-            if (gjs->should_exit(&code))
+            if (gjs->should_exit(&code)) {
+                gjs->warn_about_unhandled_promise_rejections();
                 exit(code);
+            }
 
             // Some other uncatchable exception, e.g. out of memory
             JSFunction* fn = callable();
