@@ -12,6 +12,8 @@
 #endif
 
 #include <stdbool.h>    /* IWYU pragma: keep */
+#include <stdint.h>
+#include <sys/types.h> /* for ssize_t */
 
 #ifndef _WIN32
 #    include <signal.h> /* for siginfo_t */
@@ -44,14 +46,30 @@ GJS_EXPORT GJS_USE bool gjs_context_eval_file(GjsContext* js_context,
                                               const char* filename,
                                               int* exit_status_p,
                                               GError** error);
+GJS_EXPORT GJS_USE bool gjs_context_eval_module_file(GjsContext* js_context,
+                                                     const char* filename,
+                                                     uint8_t* exit_status_p,
+                                                     GError** error);
 GJS_EXPORT GJS_USE bool gjs_context_eval(GjsContext* js_context,
                                          const char* script, gssize script_len,
                                          const char* filename,
                                          int* exit_status_p, GError** error);
+GJS_EXPORT GJS_USE bool gjs_context_register_module(GjsContext* context,
+                                                    const char* identifier,
+                                                    const char* uri,
+                                                    GError** error);
+GJS_EXPORT GJS_USE bool gjs_context_eval_module(GjsContext* context,
+                                                const char* identifier,
+                                                uint8_t* exit_code,
+                                                GError** error);
 GJS_EXPORT GJS_USE bool gjs_context_define_string_array(
     GjsContext* js_context, const char* array_name, gssize array_length,
     const char** array_values, GError** error);
 
+GJS_EXPORT void gjs_context_set_argv(GjsContext* js_context,
+                                     ssize_t array_length,
+                                     const char** array_values);
+
 GJS_EXPORT GJS_USE GList* gjs_context_get_all(void);
 
 GJS_EXPORT GJS_USE GjsContext* gjs_context_get_current(void);
