@@ -19,11 +19,10 @@
 #include <js/TypeDecls.h>
 #include <js/Utility.h>  // for UniqueChars
 #include <js/Value.h>
+#include <js/friend/DumpFunctions.h>
 #include <jsapi.h>        // for MaxFrames, CaptureCurrentStack
-#include <jsfriendapi.h>  // for FormatStackDump
 
 #include "gjs/deprecation.h"
-#include "gjs/jsapi-util.h"  // IWYU pragma: keep
 #include "gjs/macros.h"
 
 const char* messages[] = {
