@@ -39,6 +39,7 @@
 #include "gjs/importer.h"
 #include "gjs/jsapi-util-args.h"
 #include "gjs/jsapi-util.h"
+#include "gjs/mainloop.h"
 #include "gjs/mem-private.h"
 #include "gjs/module.h"
 #include "gjs/native.h"
@@ -87,17 +88,8 @@ bool gjs_load_internal_module(JSContext* cx, const char* identifier) {
     JS::RootedObject internal_global(cx, gjs_get_internal_global(cx));
     JSAutoRealm ar(cx, internal_global);
 
-    JS::RootedObject module(cx, JS::CompileModule(cx, options, buf));
-    JS::RootedObject registry(cx, gjs_get_module_registry(internal_global));
-
-    JS::RootedId key(cx, gjs_intern_string_to_id(cx, full_path));
-
-    if (!gjs_global_registry_set(cx, registry, key, module) ||
-        !JS::ModuleInstantiate(cx, module) || !JS::ModuleEvaluate(cx, module)) {
-        return false;
-    }
-
-    return true;
+    JS::RootedValue ignored(cx);
+    return JS::Evaluate(cx, options, buf, &ignored);
 }
 
 static bool handle_wrong_args(JSContext* cx) {
@@ -278,6 +270,7 @@ bool gjs_internal_parse_uri(JSContext* cx, unsigned argc, JS::Value* vp) {
     using AutoHashTable =
         GjsAutoPointer<GHashTable, GHashTable, g_hash_table_destroy>;
     using AutoURI = GjsAutoPointer<GUri, GUri, g_uri_unref>;
+    GjsContextPrivate* gjs = GjsContextPrivate::from_cx(cx);
 
     JS::CallArgs args = CallArgsFromVp(argc, vp);
     JS::RootedString string_arg(cx);
@@ -291,6 +284,8 @@ bool gjs_internal_parse_uri(JSContext* cx, unsigned argc, JS::Value* vp) {
     GError* error = nullptr;
     AutoURI parsed = g_uri_parse(uri.get(), G_URI_FLAGS_NONE, &error);
     if (!parsed) {
+        JSAutoRealm ar(cx, gjs->global());
+
         gjs_throw_custom(cx, JSProto_Error, "ImportError",
                          "Attempted to import invalid URI: %s (%s)", uri.get(),
                          error->message);
@@ -307,6 +302,8 @@ bool gjs_internal_parse_uri(JSContext* cx, unsigned argc, JS::Value* vp) {
         AutoHashTable query =
             g_uri_parse_params(raw_query, -1, "&", G_URI_PARAMS_NONE, &error);
         if (!query) {
+            JSAutoRealm ar(cx, gjs->global());
+
             gjs_throw_custom(cx, JSProto_Error, "ImportError",
                              "Attempted to import invalid URI: %s (%s)",
                              uri.get(), error->message);
@@ -409,6 +406,9 @@ bool gjs_internal_load_resource_or_file(JSContext* cx, unsigned argc,
     GError* error = nullptr;
     if (!g_file_load_contents(file, /* cancellable = */ nullptr, &contents,
                               &length, /* etag_out = */ nullptr, &error)) {
+        GjsContextPrivate* gjs = GjsContextPrivate::from_cx(cx);
+        JSAutoRealm ar(cx, gjs->global());
+
         gjs_throw_custom(cx, JSProto_Error, "ImportError",
                          "Unable to load file from: %s (%s)", uri.get(),
                          error->message);
@@ -503,7 +503,10 @@ class PromiseData {
 static void load_async_callback(GObject* file, GAsyncResult* res, void* data) {
     std::unique_ptr<PromiseData> promise(PromiseData::from_ptr(data));
 
-    JSAutoRealm ac(promise->cx, gjs_get_import_global(promise->cx));
+    GjsContextPrivate* gjs = GjsContextPrivate::from_cx(promise->cx);
+    gjs->main_loop_release();
+
+    JSAutoRealm ar(promise->cx, gjs->global());
 
     char* contents;
     size_t length;
@@ -551,6 +554,9 @@ static bool load_async_executor(JSContext* cx, unsigned argc, JS::Value* vp) {
 
     auto* data = new PromiseData(cx, JS_GetObjectFunction(resolve),
                                  JS_GetObjectFunction(reject));
+
+    // Hold the main loop until this function resolves...
+    GjsContextPrivate::from_cx(cx)->main_loop_hold();
     g_file_load_contents_async(file, nullptr, load_async_callback, data);
 
     args.rval().setUndefined();
