@@ -34,6 +34,8 @@
 
 #include "gjs/context.h"
 #include "gjs/jsapi-util.h"
+#include "gjs/mem-private.h"
+#include "gjs/profiler-private.h"
 #include "gjs/profiler.h"
 
 #define FLUSH_DELAY_SECONDS 3
@@ -104,6 +106,8 @@ struct _GjsProfiler {
 
     /* GLib signal handler ID for SIGUSR2 */
     unsigned sigusr2_id;
+    unsigned counter_base;  // index of first GObject memory counter
+    unsigned gc_counter_base;  // index of first GC stats counter
 #endif  /* ENABLE_PROFILER */
 
     /* If we are currently sampling */
@@ -164,6 +168,66 @@ static GjsContext *profiling_context;
 
     return true;
 }
+
+static void setup_counter_helper(SysprofCaptureCounter* counter,
+                                 const char* counter_name,
+                                 unsigned counter_base, size_t ix) {
+    g_snprintf(counter->category, sizeof counter->category, "GJS");
+    g_snprintf(counter->name, sizeof counter->name, "%s", counter_name);
+    g_snprintf(counter->description, sizeof counter->description, "%s",
+               GJS_COUNTER_DESCRIPTIONS[ix]);
+    counter->id = uint32_t(counter_base + ix);
+    counter->type = SYSPROF_CAPTURE_COUNTER_INT64;
+    counter->value.v64 = 0;
+}
+
+[[nodiscard]] static bool gjs_profiler_define_counters(GjsProfiler* self) {
+    int64_t now = g_get_monotonic_time() * 1000L;
+
+    g_assert(self && "Profiler must be set up before defining counters");
+
+    SysprofCaptureCounter counters[GJS_N_COUNTERS];
+    self->counter_base =
+        sysprof_capture_writer_request_counter(self->capture, GJS_N_COUNTERS);
+
+#    define SETUP_COUNTER(counter_name, ix)                                    \
+        setup_counter_helper(&counters[ix], #counter_name, self->counter_base, \
+                             ix);
+    GJS_FOR_EACH_COUNTER(SETUP_COUNTER);
+#    undef SETUP_COUNTER
+
+    if (!sysprof_capture_writer_define_counters(
+            self->capture, now, -1, self->pid, counters, GJS_N_COUNTERS))
+        return false;
+
+    SysprofCaptureCounter gc_counters[Gjs::GCCounters::N_COUNTERS];
+    self->gc_counter_base = sysprof_capture_writer_request_counter(
+        self->capture, Gjs::GCCounters::N_COUNTERS);
+
+    constexpr size_t category_size = sizeof gc_counters[0].category;
+    constexpr size_t name_size = sizeof gc_counters[0].name;
+    constexpr size_t description_size = sizeof gc_counters[0].description;
+
+    for (size_t ix = 0; ix < Gjs::GCCounters::N_COUNTERS; ix++) {
+        g_snprintf(gc_counters[ix].category, category_size, "GJS");
+        gc_counters[ix].id = uint32_t(self->gc_counter_base + ix);
+        gc_counters[ix].type = SYSPROF_CAPTURE_COUNTER_INT64;
+        gc_counters[ix].value.v64 = 0;
+    }
+    g_snprintf(gc_counters[Gjs::GCCounters::GC_HEAP_BYTES].name, name_size,
+               "GC bytes");
+    g_snprintf(gc_counters[Gjs::GCCounters::GC_HEAP_BYTES].description,
+               description_size, "Bytes used in GC heap");
+    g_snprintf(gc_counters[Gjs::GCCounters::MALLOC_HEAP_BYTES].name, name_size,
+               "Malloc bytes");
+    g_snprintf(gc_counters[Gjs::GCCounters::MALLOC_HEAP_BYTES].description,
+               description_size, "Malloc bytes owned by tenured GC things");
+
+    return sysprof_capture_writer_define_counters(self->capture, now, -1,
+                                                  self->pid, gc_counters,
+                                                  Gjs::GCCounters::N_COUNTERS);
+}
+
 #endif  /* ENABLE_PROFILER */
 
 /*
@@ -364,7 +428,22 @@ static void gjs_profiler_sigprof(int signum [[maybe_unused]], siginfo_t* info,
     }
 
     if (!sysprof_capture_writer_add_sample(self->capture, now, -1, self->pid,
-                                           -1, addrs, depth))
+                                           -1, addrs, depth)) {
+        gjs_profiler_stop(self);
+        return;
+    }
+
+    unsigned ids[GJS_N_COUNTERS];
+    SysprofCaptureCounterValue values[GJS_N_COUNTERS];
+
+#    define FETCH_COUNTERS(name, ix)       \
+        ids[ix] = self->counter_base + ix; \
+        values[ix].v64 = GJS_GET_COUNTER(name);
+    GJS_FOR_EACH_COUNTER(FETCH_COUNTERS);
+#    undef FETCH_COUNTERS
+
+    if (!sysprof_capture_writer_set_counters(self->capture, now, -1, self->pid,
+                                             ids, values, GJS_N_COUNTERS))
         gjs_profiler_stop(self);
 }
 
@@ -451,6 +530,13 @@ gjs_profiler_start(GjsProfiler *self)
         return;
     }
 
+    if (!gjs_profiler_define_counters(self)) {
+        g_warning("Failed to define sysprof counters");
+        g_clear_pointer(&self->capture, sysprof_capture_writer_unref);
+        g_clear_pointer(&self->periodic_flush, g_source_destroy);
+        return;
+    }
+
     /* Setup our signal handler for SIGPROF delivery */
     sa.sa_flags = SA_RESTART | SA_SIGINFO;
     sa.sa_sigaction = gjs_profiler_sigprof;
@@ -720,6 +806,33 @@ void _gjs_profiler_add_mark(GjsProfiler* self, gint64 time_nsec,
 #endif
 }
 
+bool _gjs_profiler_sample_gc_memory_info(
+    GjsProfiler* self, int64_t gc_counters[Gjs::GCCounters::N_COUNTERS]) {
+    g_return_val_if_fail(self, false);
+
+#ifdef ENABLE_PROFILER
+    if (self->running && self->capture) {
+        unsigned ids[Gjs::GCCounters::N_COUNTERS];
+        SysprofCaptureCounterValue values[Gjs::GCCounters::N_COUNTERS];
+
+        for (size_t ix = 0; ix < Gjs::GCCounters::N_COUNTERS; ix++) {
+            ids[ix] = self->gc_counter_base + ix;
+            values[ix].v64 = gc_counters[ix];
+        }
+
+        int64_t now = g_get_monotonic_time() * 1000L;
+        if (!sysprof_capture_writer_set_counters(self->capture, now, -1,
+                                                 self->pid, ids, values,
+                                                 Gjs::GCCounters::N_COUNTERS))
+            return false;
+    }
+#else
+    // Unused in the no-profiler case
+    (void)gc_counters;
+#endif
+    return true;
+}
+
 void gjs_profiler_set_fd(GjsProfiler* self, int fd) {
     g_return_if_fail(self);
     g_return_if_fail(!self->filename);
