@@ -1,5 +1,8 @@
+// SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+// SPDX-FileCopyrightText: 2008 litl, LLC
+
 const ByteArray = imports.byteArray;
-const {Gio, GjsPrivate, GLib} = imports.gi;
+const {Gio, GjsTestTools, GLib} = imports.gi;
 
 /* The methods list with their signatures.
  *
@@ -139,6 +142,10 @@ class Test {
         return `${a} ${b} ${c} ${d} ${e}`;
     }
 
+    emitPropertyChanged(name, value) {
+        this._impl.emit_property_changed(name, value);
+    }
+
     emitSignal() {
         this._impl.emit_signal('signalFoo', GLib.Variant.new('(s)', ['foobar']));
     }
@@ -237,14 +244,14 @@ class Test {
     }
 
     fdOut(bytes) {
-        const fd = GjsPrivate.open_bytes(bytes);
+        const fd = GjsTestTools.open_bytes(bytes);
         const fdList = Gio.UnixFDList.new_from_array([fd]);
         return [0, fdList];
     }
 
     fdOut2Async([bytes], invocation) {
         GLib.idle_add(GLib.PRIORITY_DEFAULT, function () {
-            const fd = GjsPrivate.open_bytes(bytes);
+            const fd = GjsTestTools.open_bytes(bytes);
             const fdList = Gio.UnixFDList.new_from_array([fd]);
             invocation.return_value_with_unix_fd_list(new GLib.Variant('(h)', [0]),
                 fdList);
@@ -553,7 +560,7 @@ describe('Exported DBus object', function () {
 
     it('can call a remote method with a Unix FD', function (done) {
         const expectedBytes = ByteArray.fromString('some bytes');
-        const fd = GjsPrivate.open_bytes(expectedBytes);
+        const fd = GjsTestTools.open_bytes(expectedBytes);
         const fdList = Gio.UnixFDList.new_from_array([fd]);
         proxy.fdInRemote(0, fdList, ([bytes], exc, outFdList) => {
             expect(exc).toBeNull();
@@ -565,7 +572,7 @@ describe('Exported DBus object', function () {
 
     it('can call an asynchronously implemented remote method with a Unix FD', function (done) {
         const expectedBytes = ByteArray.fromString('some bytes');
-        const fd = GjsPrivate.open_bytes(expectedBytes);
+        const fd = GjsTestTools.open_bytes(expectedBytes);
         const fdList = Gio.UnixFDList.new_from_array([fd]);
         proxy.fdIn2Remote(0, fdList, ([bytes], exc, outFdList) => {
             expect(exc).toBeNull();
@@ -658,4 +665,21 @@ describe('Exported DBus object', function () {
 
         expect(proxy.PropReadOnly).toBe(PROP_READ_ONLY_INITIAL_VALUE);
     });
+
+    it('Marking a property as invalidated works', function () {
+        let changedProps = {};
+        let invalidatedProps = [];
+
+        proxy.connect('g-properties-changed', (proxy_, changed, invalidated) => {
+            changedProps = changed.deepUnpack();
+            invalidatedProps = invalidated;
+            loop.quit();
+        });
+
+        test.emitPropertyChanged('PropReadOnly', null);
+        loop.run();
+
+        expect(changedProps).not.toContain('PropReadOnly');
+        expect(invalidatedProps).toContain('PropReadOnly');
+    });
 });
