@@ -276,12 +276,10 @@ static bool region_release_argument(JSContext*, GITransfer transfer,
     return true;
 }
 
-static GjsForeignInfo foreign_info = {
-    region_to_g_argument,
-    region_from_g_argument,
-    region_release_argument
-};
 
 void gjs_cairo_region_init(void) {
+    static GjsForeignInfo foreign_info = {
+        region_to_g_argument, region_from_g_argument, region_release_argument};
+
     gjs_struct_foreign_register("cairo", "Region", &foreign_info);
 }
