@@ -80,8 +80,6 @@ FALSE_POSITIVES = (
     # https://bugzilla.mozilla.org/show_bug.cgi?id=1713550
     ('gi/function.cpp', '#include <utility>', 'for forward'),
     ('gi/ns.cpp', '#include <utility>', 'for forward'),
-    ('gi/value.cpp', '#include <utility>', 'for forward'),
-    ('gjs/importer.cpp', '#include <utility>', 'for forward'),
     ('gjs/module.cpp', '#include <utility>', 'for forward'),
     ('gjs/objectbox.cpp', '#include <utility>', 'for forward'),
 
@@ -95,6 +93,8 @@ FALSE_POSITIVES = (
     # False positive when using GjsAutoPointer
     # https://github.com/include-what-you-use/include-what-you-use/issues/927
     ('gi/boxed.cpp', '#include <type_traits>', 'for remove_reference<>::type'),
+    ('gi/interface.cpp', '#include <type_traits>',
+     'for remove_reference<>::type'),
     ('gi/object.cpp', '#include <type_traits>',
      'for remove_reference<>::type'),
     ('gi/private.cpp', '#include <type_traits>',
@@ -108,6 +108,8 @@ FALSE_POSITIVES = (
      'for remove_reference<>::type'),
     ('gjs/profiler.cpp', '#include <type_traits>',
      'for remove_reference<>::type'),
+    ('gjs/promise.cpp', '#include <type_traits>',
+     'for remove_reference<>::type'),
     ('test/gjs-test-jsapi-utils.cpp', '#include <type_traits>',
      'for remove_reference<>::type'),
 
